#include "FT_Transport_SPI.h"
#include "FT811.h"

#define FT_CS_PIN           10
#define FT_PDN_PIN          8
#define FT_INT_PIN          9
#define FT_DISPENABLE_PIN   7
#define FT_AUDIOENABLE_PIN  1

FT_Transport_SPI spi;

int CmdFifoWp = 0;
uint8_t dev_id = 0, Timout = 0;
uint16_t CmdOffset = 0, CmdBufferRead = 0;
uint32_t addr = 0;

void HostCommand(uint32_t HostCommand)
{
  uint32_t Addr;
  /* construct host command and send to graphics controller */
  Addr = HostCommand << 16;
  spi.Read(Addr);//ideally sending 3 bytes is sufficient
  delay(20);//worst scenario
}

char TransferCmd(uint32_t Cmd)
{
  spi.StartWrite(FT_RAM_CMD + CmdFifoWp);
  spi.Transfer32(Cmd);
  CmdFifoWp = (CmdFifoWp + 4) & 0xfff;
  spi.EndTransfer();
  spi.Write32(REG_CMD_WRITE, CmdFifoWp);
}

void setup() {

  pinMode(FT_DISPENABLE_PIN, OUTPUT);
  digitalWrite(FT_DISPENABLE_PIN, HIGH);
  pinMode(FT_PDN_PIN, OUTPUT);
  pinMode(FT_INT_PIN, INPUT);
  digitalWrite(FT_PDN_PIN, HIGH);

  spi.SetCSpin(FT_CS_PIN);
  spi.Init();

  digitalWrite(FT_PDN_PIN, LOW);
  delay(25);
  digitalWrite(FT_PDN_PIN, HIGH);
  delay(50);

  HostCommand(FT_CLKEXT);
  HostCommand(FT_ACTIVE);//wake up the processor from sleep state
  delay(20);

  dev_id = spi.Read(REG_ID);
  while (dev_id != 0x7C)
  {
    dev_id = spi.Read(REG_ID);
    delay(5);
    if (++Timout >= 200) return 0;
  }

  CmdOffset = spi.Read16(REG_CMD_WRITE);
  addr = (FT_RAM_CMD + CmdOffset);
  spi.Writefromflash(addr, GT911Data, 1216);

  CmdOffset = 1216;
  spi.Write16(REG_CMD_WRITE, CmdOffset);


  while (CmdBufferRead != CmdOffset)
  {
    CmdBufferRead = spi.Read16(REG_CMD_READ);
    delayMicroseconds(25);
  }

  spi.Write(REG_CPURESET, 0x02);          // Touch Engine
  spi.Write(REG_TOUCH_OVERSAMPLE, 0x0F);  //set touch oversampling to max
  spi.Write16(REG_TOUCH_CONFIG, 0x05D0);  // CTE
  spi.Write16(REG_GPIOX_DIR, 0x8001);     // Low output on GPIO0
  delay(1);
  spi.Write(REG_CPURESET, 0x00);          // Coprocessor Engine
  delay(56);
  spi.Write16(REG_GPIOX_DIR, 0x8000);     // Input GPIO0
  //*********************************************************************************************************

  spi.Write16(REG_HCYCLE,     FT_DISPLAY_HCYCLE_WVGA );
  spi.Write16(REG_HOFFSET,    FT_DISPLAY_HOFFSET_WVGA);
  spi.Write16(REG_HSYNC0,     FT_DISPLAY_HSYNC0_WVGA );
  spi.Write16(REG_HSYNC1,     FT_DISPLAY_HSYNC1_WVGA );
  spi.Write16(REG_VCYCLE,     FT_DISPLAY_VCYCLE_WVGA );
  spi.Write16(REG_VOFFSET,    FT_DISPLAY_VOFFSET_WVGA);
  spi.Write16(REG_VSYNC0,     FT_DISPLAY_VSYNC0_WVGA );
  spi.Write16(REG_VSYNC1,     FT_DISPLAY_VSYNC1_WVGA );
  spi.Write16(REG_SWIZZLE,    FT_DISPLAY_SWIZZLE_WVGA);
  spi.Write16(REG_PCLK_POL,   FT_DISPLAY_PCLKPOL_WVGA);
  spi.Write16(REG_CSPREAD,  0);

  spi.Write16(REG_HSIZE,    FT_DISPLAY_HSIZE_WVGA );
  spi.Write16(REG_VSIZE,    FT_DISPLAY_VSIZE_WVGA );

  spi.Write(REG_CTOUCH_MODE, 0x03);      // On Mode
  spi.Write16(REG_TOUCH_RZTHRESH, 5000);  // set touch resistance threshold

  spi.Write32(FT_RAM_DL, CLEAR_COLOR_RGB(0, 0, 0));
  spi.Write32(FT_RAM_DL + 4, CLEAR(1, 1, 1));
  spi.Write32(FT_RAM_DL + 8, DISPLAY());

  spi.Write(REG_GPIO, 0x80);

  spi.Write(REG_DLSWAP, FT_DLSWAP_FRAME);

  spi.Write16(REG_PCLK, FT_DISPLAY_PCLK_WVGA);

  TransferCmd(CMD_LOGO);
}

void loop() {
  // put your main code here, to run repeatedly:

}
